"""
URLs da API
"""
from django.conf.urls import url, include
from rest_framework.urlpatterns import format_suffix_patterns
from rest_framework.schemas import get_schema_view
from contas_orama import views

SCHEMA_VIEW = get_schema_view(title="Contas API")

urlpatterns = [
    url(r'^contas/$', views.ContaList.as_view(), name='conta-list'),
    url(r'^contas/(?P<pk>[0-9]+)/$',
        views.ContaDetail.as_view(), name='conta-detail'),
    url(r'^contas/(?P<pk>[0-9]+)/novo-lancamento/$',
        views.LancamentoCreate.as_view(),
        name='lancamento-create'),
    url(r'^lancamento/(?P<pk>[0-9]+)$',
        views.LancamentoDetail.as_view(),
        name='lancamento-detail'),
    url(r'^contas/(?P<pk>[0-9]+)/lancamentos/(?P<ini>.+)/(?P<end>.+)/$',
        views.LancamentoList.as_view(),
        name='lancamento-list'),
    url(r'^$', views.api_root),
    url(r'^usuarios/$', views.UserList.as_view(), name='user-list'),
    url(r'^usuarios/(?P<pk>[0-9]+)/$', views.UserDetail.as_view(),
        name='user-detail'),
]

# URL PARA O LOGIN DO USUÁRIO
urlpatterns += [
    url(r'^api-auth/', include('rest_framework.urls'), name='login')
]

urlpatterns = format_suffix_patterns(urlpatterns)



"""
Models da API
"""
import datetime
import decimal

from django.db import models


class Conta(models.Model):
    """
    Contas dos Usuários
    Ex.: Conta corrente, Investimentos, Cartão de Crédito, etc...
    """
    name = models.CharField('Título da Conta:',
                            max_length=255)
    description = models.TextField('Descrição:',
                                   blank=True,
                                   default='')
    owner = models.ForeignKey('auth.User',
                              related_name='contas',
                              on_delete=models.CASCADE)

    def __str__(self):
        """
        Faz com que a conta string de conta seja igual a
        field name
        """
        return self.name

    class Meta:
        ordering = ["name"]


class LancamentoManager(models.Manager):
    """
    Manager de Lancamento
    Criado para gerar a função get_balance
    """
    def get_balance(self, date_end, conta_id, date_ini=None):
        """
        Função get_balance retorna o saldo durante o período entre date_ini e
        date_end da conta com o id igual a conta_id.

        Caso não seja dado o parâmetro date_ini, o saldo é calculado para
        desde o primeiro lançamento na conta, até o dia anterior ao date_end.
        """
        if date_ini is None:
            lanc_filt_date = self.filter(date__lt=date_end,
                                         conta=conta_id)
        else:
            lanc_filt_date = self.filter(date__gte=date_ini,
                                         date__lte=date_end,
                                         conta=conta_id)

        # Soma dos débitos
        deb_sum = lanc_filt_date.filter(
            lanc_tp=1).aggregate(models.Sum('value'))
        if deb_sum['value__sum'] is None:
            deb_sum['value__sum'] = decimal.Decimal(0.00)

        # Soma dos créditos
        cre_sum = lanc_filt_date.filter(
            lanc_tp=2).aggregate(models.Sum('value'))
        if cre_sum['value__sum'] is None:
            cre_sum['value__sum'] = decimal.Decimal(0.00)

        return deb_sum['value__sum'] - cre_sum['value__sum']


class Lancamento(models.Model):
    """
    Lancamento dos Usuários
    Ex.: Salário, Compra de Ativos, Padaria, etc...
    """
    Transactions_Choices = (
        (1, 'Débito'),
        (2, 'Crédito'),
    )

    name = models.CharField('Título do Lançamento:',
                            max_length=255)
    date = models.DateField('Data do Lançamento:',
                            default=datetime.date.today)
    lanc_tp = models.IntegerField('Tipo de Lançamento:',
                                  choices=Transactions_Choices,
                                  default=1)
    value = models.DecimalField('Valor do Lançamento:',
                                max_digits=12,
                                decimal_places=2)
    conta = models.ForeignKey(Conta,
                              related_name='lancamentos',
                              on_delete=models.CASCADE)
    owner = models.ForeignKey('auth.User',
                              related_name='lancamentos',
                              on_delete=models.CASCADE)
    objects = LancamentoManager()



"""
Serializers a serem utilizadas em contas/views.py
"""
from django.contrib.auth.models import User
from rest_framework import serializers

from contas_orama.models import Conta, Lancamento


class ContaSerializer(serializers.HyperlinkedModelSerializer):
    """
    Serializer para preencher novas contas e vincular aos donos.
    """
    owner = serializers.ReadOnlyField(source='owner.username')
    lancamentos = serializers.HyperlinkedRelatedField(
        many=True, view_name='lancamento-detail', read_only=True)

    class Meta:
        model = Conta
        fields = ('url', 'id', 'name', 'description', 'owner', 'lancamentos')


class LancamentoSerializer(serializers.HyperlinkedModelSerializer):
    """
    Serializer para preencher novos lançamentos e vincular
    contas e donos.
    """
    conta = serializers.ReadOnlyField(source='conta.id')
    owner = serializers.ReadOnlyField(source='owner.username')

    class Meta:
        model = Lancamento
        fields = ('url', 'id', 'name', 'value', 'date', 'lanc_tp',
                  'conta', 'owner')


class DateFilterSerializer(serializers.Serializer):
    """
    Serializer para preencher datas que filtrarão os lançamentos.
    De modo a formar um extrato.
    """
    def validate(self, data):
        """
        Função para se certificar que a data inicial é anterior a data final
        """
        if data['data_ini'] > data['data_end']:
            raise serializers.ValidationError("Data final deve ser posterior \
                                               a data inicial")
        return data

    data_ini = serializers.DateField('Data Inicial:')
    data_end = serializers.DateField('Data Final:')


class UserSerializer(serializers.HyperlinkedModelSerializer):
    """
    Serializer para preencher novos usuários e vincular as
    contas aos mesmos
    """
    contas = serializers.HyperlinkedRelatedField(many=True,
                                                 view_name='conta-detail',
                                                 read_only=True)

    class Meta:
        model = User
        fields = ('url', 'id', 'username', 'contas')



"""
Permissões a serem utilizadas em views.py
"""
from rest_framework import permissions


class IsOwner(permissions.BasePermission):
    """
    Permissão para que somente os donos da conta/lancamento
    possam visualiza-los e/ou altera-los
    """
    def has_object_permission(self, request, view, obj):
        return obj.owner == request.user


class IsUser(permissions.BasePermission):
    """
    Permissão para que somente os próprios usuários possam
    visualiza-los e/ou altera-los.
    """
    def has_object_permission(self, request, view, obj):
        return obj.username == request.user.username



"""
Testes a serem utilizadas em views.py
"""
import subprocess
import os

from rest_framework import status
from rest_framework.test import APITestCase

from django.urls import reverse
from django.contrib.auth.models import User

from contas_orama.models import Conta, Lancamento

# Obtém o diretório onde a API contas_orama foi salva
CONTAS_ORAMA_DIR = os.path.join(os.path.dirname(
    os.path.dirname(os.path.abspath(__file__))), 'contas_orama')

# Testa o pylint
# Exceções: W0221, W0223, R0901, R0903
subprocess.call('pylint --load-plugins pylint_django \
                --disable=W0221,W0223,R0901,R0903 ' + CONTAS_ORAMA_DIR,
                shell=True)

# Testa o pycodestyle
subprocess.call('pycodestyle ' + CONTAS_ORAMA_DIR, shell=True)


class ContaListAllTest(APITestCase):
    """
    Módulo de teste para a listagem de todas as contas.
    Testa o ContaList view em contas/views.py
    """
    def setUp(self):
        """
        Cria o banco necessário para rodar os testes
        """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        # Cria contas
        Conta.objects.create(
            name="Conta Corrente",
            description="Transações da conta corrente do banco: ABC",
            owner=User.objects.get(username='joao'))
        Conta.objects.create(
            name="Investimentos",
            description="Investimentos feito pela corretora: XYZ",
            owner=User.objects.get(username='joao'))

    def test_get_all_contas(self):
        """
        Verifica se está listando as contas corretamente
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('conta-list'), format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Conta.objects.count(), 2)


class ContaGetUnicTest(APITestCase):
    """
    Módulo de teste para selecionar uma única conta.
    Testa o ContaDetail view em contas/views.py
    """
    def setUp(self):
        """
        Cria o banco necessário para rodar os testes
        """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        self.contacorrente = Conta.objects.create(
            name="Conta Corrente",
            description="Transações da conta corrente do banco: ABC",
            owner=User.objects.get(username='joao'))
        self.cartaocredito = Conta.objects.create(
            name="Cartão de Crédito",
            description="Fatura do cartão de crédito",
            owner=User.objects.get(username='joao'))
        self.investimentos = Conta.objects.create(
            name="Investimentos",
            description="Investimentos feito pela corretora: XYZ",
            owner=User.objects.get(username='maria'))

    def test_get_valid_conta(self):
        """
        Verifica se está selecionando as contas corretamente
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('conta-detail',
                                           kwargs={'pk':
                                                   self.cartaocredito.pk}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Conta.objects.get(pk=self.cartaocredito.pk).name,
                         "Cartão de Crédito")

    def test_get_invalid_conta(self):
        """
        Verifica se está retornando 404 quando se
        seleciona objetos não existentes
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('conta-detail',
                                           kwargs={'pk': 100}))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_forbidden_conta(self):
        """
        Verifica se está bloqueando a seleção por usuários sem
        permissão.
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('conta-detail',
                                           kwargs={'pk':
                                                   self.investimentos.pk}))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class ContaNewTest(APITestCase):
    """
    Módulo de teste para a criação de uma nova conta.
    Testa o ContaList view em contas/views.py
    """
    def setUp(self):
        """
        Cria o banco necessário para rodar os testes
        """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        # requests de contas
        self.valid_payload = {
            'name': 'Conta',
            'description': 'Descrição da conta.'
        }
        self.invalid_payload = {
            'name': '',
            'description': ''
        }

    def test_create_valid_conta(self):
        """
        Verifica se estão sendo criados objetos corretamente
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(reverse('conta-list'),
                                    self.valid_payload,
                                    format='json')
        self.assertEqual(response.status_code, status.HTTP_302_FOUND)
        self.assertEqual(Conta.objects.count(), 1)
        self.assertEqual(Conta.objects.get().owner.username, 'joao')
        self.assertEqual(Conta.objects.get().name, 'Conta')

    def test_create_invalid_conta(self):
        """
        Verifica se está retornando má requisição se estiver
        tentando criar objetos não permitidos.
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(reverse('conta-list'),
                                    self.invalid_payload,
                                    format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_forbiden_cont(self):
        """
        Verifica se está bloqueando a tentativa de criar
        objetos por usuários não logados.
        """
        response = self.client.post(reverse('conta-list'),
                                    self.valid_payload,
                                    format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class ContaUpdateTest(APITestCase):
    """
    Módulo de teste para a atualização de uma conta existente.
    Testa o ContaDetail view em contas/views.py
    """
    def setUp(self):
        """
        Cria o banco necessário para rodar os testes
        """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        self.investimentos = Conta.objects.create(
            name="Investimentos",
            description="Investimentos feito pela corretora: XYZ",
            owner=User.objects.get(username='maria'))
        # requests de contas
        self.valid_payload = {
            'name': 'Conta',
            'description': 'Descrição da conta.'
        }
        self.invalid_payload = {
            'name': '',
            'description': ''
        }

    def test_update_valid_conta(self):
        """
        Verifica se estão sendo atualizados objetos corretamente
        """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.put(
            reverse('conta-detail', kwargs={'pk': self.investimentos.pk}),
            self.valid_payload, format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Conta.objects.get(pk=self.investimentos.pk).name,
                         'Conta')

    def test_update_invalid_conta(self):
        """
        Verifica se está retornando má requisição se estiver tentando
        atualizar objetos de maneira não permitida
        """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.put(
            reverse('conta-detail', kwargs={'pk': self.investimentos.pk}),
            self.invalid_payload, format='json')

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_update_not_allowed_conta(self):
        """
        Verifica se está bloqueando a tentativa de atualizar
        objetos por usuários não permitidos
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(
            reverse('conta-detail', kwargs={'pk': self.investimentos.pk}),
            self.valid_payload, format='json')

        self.assertEqual(response.status_code,
                         status.HTTP_405_METHOD_NOT_ALLOWED)

    def test_update_forbiden_cont(self):
        """
        Verifica se está bloqueando a tentativa de atualizar
        objetos por usuários não logados
        """
        response = self.client.post(
            reverse('conta-detail', kwargs={'pk': self.investimentos.pk}),
            self.valid_payload, format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class ContaDeleteTest(APITestCase):
    """
    Módulo para deletar uma conta existente.
    Testa o ContaDetail view em contas/views.py
    """
    def setUp(self):
        """
        Cria o banco necessário para rodar os testes
        """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        self.investimentos = Conta.objects.create(
            name="Investimentos",
            description="Investimentos feito pela corretora: XYZ",
            owner=User.objects.get(username='maria'))

    def test_delete_conta_valid(self):
        """
        Verifica se os objetos estão sendo deletados
        corretamente
        """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.delete(
            reverse('conta-detail',
                    kwargs={'pk': self.investimentos.pk}))
        self.assertEqual(response.status_code,
                         status.HTTP_204_NO_CONTENT)

    def test_delete_conta_invalid(self):
        """
        Verifica se está retornando 404 quando se seleciona objetos
        não existentes
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.delete(
            reverse('conta-detail', kwargs={'pk': 20}))

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_not_allowed_conta(self):
        """
        Verifica se está bloqueando a tentativa de deletar
        objetos por usuários não permitidos
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.delete(
            reverse('conta-detail',
                    kwargs={'pk': self.investimentos.pk}))

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_forbiden_cont(self):
        """
        Verifica se está bloqueando a tentativa de deletar
        objetos por usuários não logados
        """
        response = self.client.delete(
            reverse('conta-detail',
                    kwargs={'pk': self.investimentos.pk}))

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class LancamentoRetrieveContaTest(APITestCase):
    """
    Módulo de teste para selecionar uma única conta para fazer um
    novo lançamento.
    Testa o LancamentoCreate view em contas/views.py
    """
    def setUp(self):
        """
        Cria o banco necessário para rodar os testes
        """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        self.cartaocredito = Conta.objects.create(
            name="Cartão de Crédito",
            description="Fatura do cartão de crédito",
            owner=User.objects.get(username='joao'))
        self.investimentos = Conta.objects.create(
            name="Investimentos",
            description="Investimentos feito pela corretora: XYZ",
            owner=User.objects.get(username='maria'))

    def test_get_valid_conta(self):
        """
        Verifica se está selecionando as contas corretamente
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(
            reverse('lancamento-create', kwargs={'pk':
                                                 self.cartaocredito.pk}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Conta.objects.get(pk=self.cartaocredito.pk).name,
                         "Cartão de Crédito")

    def test_get_invalid_conta(self):
        """
        Verifica se está retornando 404 quando se seleciona objetos
        não existentes
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('lancamento-create',
                                           kwargs={'pk': 100}))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_forbidden_conta(self):
        """
        Verifica se está bloqueando a seleção por usuários sem permissão
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(
            reverse('lancamento-create', kwargs={'pk':
                                                 self.investimentos.pk}))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class LancamentoNewTest(APITestCase):
    """
    Módulo de teste para a criação de um novo lançamento.
    Testa o LancamentoCreate view em lancamentos/views.py
    """
    def setUp(self):
        """
        Cria o banco necessário para rodar os testes
        """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        Conta.objects.create(
            name="Conta Corrente",
            description="",
            owner=User.objects.get(username='joao'))
        # requests de lancamentos
        self.valid_payload = {
            'name': 'Salário',
            'value': 10.00
        }
        self.invalid_payload = {
            'name': '',
            'value': 10000.00
        }

    def test_create_valid_conta(self):
        """ Verifica se estão sendo criados objetos corretamente """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(
            reverse('lancamento-create',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk}),
            self.valid_payload,
            format='json')

        self.assertEqual(response.status_code, status.HTTP_302_FOUND)
        self.assertEqual(Lancamento.objects.count(), 1)
        self.assertEqual(Lancamento.objects.get().owner.username, 'joao')
        self.assertEqual(Lancamento.objects.get().conta.name, 'Conta Corrente')
        self.assertEqual(Lancamento.objects.get().name, 'Salário')

    def test_create_invalid_conta(self):
        """
        Verifica se está retornando má requisição se estiver tentando criar
        objetos não permitidos
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(
            reverse('lancamento-create',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk}),
            self.invalid_payload,
            format='json')

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_not_login_conta(self):
        """
        Verifica se está bloqueando a tentativa de criar objetos por usuários
        não logados
        """
        response = self.client.post(
            reverse('lancamento-create',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk}),
            self.valid_payload,
            format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_create_forbiden_conta(self):
        """
        Verifica se está bloqueando a tentativa de criar objetos por usuários
        não permitidos
        """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.post(
            reverse('lancamento-create',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk}),
            self.valid_payload,
            format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class LancamentoGetUnicTest(APITestCase):
    """
    Módulo de teste para selecionar um único lançamento.
    Testa o LancamentoDetail view em contas/views.py
    """
    def setUp(self):
        """ Cria o banco necessário para rodar os testes """
        # Criar usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria Contas
        Conta.objects.create(
            name="Conta Corrente",
            description="Transações da conta corrente do banco: ABC",
            owner=User.objects.get(username='joao'))
        Conta.objects.create(
            name="Conta Corrente",
            description="Transações da conta corrente do banco: 123",
            owner=User.objects.get(username='maria'))
        # Cria Lançamentos
        self.padaria = Lancamento.objects.create(
            name='Padaria', lanc_tp=2, value=10.00,
            owner=User.objects.get(username='joao'),
            conta=Conta.objects.get(name='Conta Corrente',
                                    owner__username='joao'))
        self.salario = Lancamento.objects.create(
            name='Salário', value=10000.00,
            owner=User.objects.get(username='maria'),
            conta=Conta.objects.get(name='Conta Corrente',
                                    owner__username='maria'))

    def test_get_valid_lancamento(self):
        """ Verifica se está selecionando os lançamentos corretamente """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('lancamento-detail',
                                           kwargs={'pk': self.padaria.pk}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Lancamento.objects.get(pk=self.padaria.pk).name,
                         "Padaria")

    def test_get_invalid_lancamento(self):
        """ Verifica se está retornando 404 quando se seleciona objetos
        não existentes """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('conta-detail',
                                           kwargs={'pk': 100}))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_forbidden_lancamento(self):
        """ Verifica se está bloqueando a seleção por usuários sem permissão
        """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('lancamento-detail',
                                           kwargs={'pk': self.salario.pk}))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class LancamentoUpdateTest(APITestCase):
    """
    Módulo de teste para a atualização de um lancamento existente.
    Testa o LancamentoDetail view em lancamentos/views.py
    """
    def setUp(self):
        """ Cria o banco necessário para rodar os testes """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        Conta.objects.create(
            name="Investimentos", description='',
            owner=User.objects.get(username='maria'))
        # Cria lançamentos
        self.ativo = Lancamento.objects.create(
            name="Compra de Ativo", lanc_tp=2, value=1000.00,
            owner=User.objects.get(username='maria'),
            conta=Conta.objects.get(name='Investimentos'))
        # requests de lançamentos
        self.valid_payload = {
            'name': 'Compra',
            'value': 10000.00
        }
        self.invalid_payload = {
            'name': '',
            'description': 0.00
        }

    def test_update_valid_lancamento(self):
        """ Verifica se estão sendo atualizados objetos corretamente """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.put(reverse('lancamento-detail',
                                           kwargs={'pk': self.ativo.pk}),
                                   self.valid_payload,
                                   format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Lancamento.objects.get(pk=self.ativo.pk).name,
                         'Compra')

    def test_update_invalid_lancamento(self):
        """ Verifica se está retornando má requisição se estiver
        tentando atualizar objetos de maneira não permitida """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.put(reverse('lancamento-detail',
                                           kwargs={'pk': self.ativo.pk}),
                                   self.invalid_payload,
                                   format='json')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_update_not_allowed_conta(self):
        """ Verifica se está bloqueando a tentativa de atualizar objetos
        por usuários não permitidos """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(reverse('lancamento-detail',
                                            kwargs={'pk': self.ativo.pk}),
                                    self.valid_payload,
                                    format='json')
        self.assertEqual(response.status_code,
                         status.HTTP_405_METHOD_NOT_ALLOWED)

    def test_update_forbiden_conta(self):
        """ Verifica se está bloqueando a tentativa de atualizar objetos
        por usuários não logados """
        response = self.client.post(reverse('lancamento-detail',
                                            kwargs={'pk': self.ativo.pk}),
                                    self.valid_payload,
                                    format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class LancamentoDeleteTest(APITestCase):
    """
    Módulo para testar o delete de um lançamento existente.
    Testa o LancamentoDetail view em lancamentos/views.py
    """
    def setUp(self):
        """ Cria o banco necessário para rodar os testes """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        Conta.objects.create(
            name="Investimentos",
            description="Detalhamento investimentos feito pela corretora: XYZ",
            owner=User.objects.get(username='maria'))
        # Cria lançamentos
        self.ativo = Lancamento.objects.create(
            name="Compra de Ativo", lanc_tp=2, value=1000.00,
            owner=User.objects.get(username='maria'),
            conta=Conta.objects.get(name='Investimentos'))

    def test_delete_valid_lancamento(self):
        """ Verifica se os objetos estão sendo deletados corretamente """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.delete(reverse('lancamento-detail',
                                              kwargs={'pk': self.ativo.pk}))
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def test_delete_invalid_lancamento(self):
        """ Verifica se está retornando 404 quando se seleciona objetos não
        existentes """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.delete(reverse('lancamento-detail',
                                              kwargs={'pk': 20}))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_forbiden_lancamento(self):
        """ Verifica se está bloqueando a tentativa de deletar objetos por
        usuários não permitidos """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.delete(reverse('lancamento-detail',
                                              kwargs={'pk': self.ativo.pk}))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_not_log_lancamento(self):
        """ Verifica se está bloqueando a tentativa de deletar objetos por
        usuários não logados """
        response = self.client.delete(reverse('conta-detail',
                                              kwargs={'pk': self.ativo.pk}))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class LancamentoListTest(APITestCase):
    """
    Módulo de teste para a listagem dos lancamentos.
    Testa o LancamentoList view em lancamento/views.py
    """
    def setUp(self):
        """ Cria o banco necessário para rodar os testes """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        Conta.objects.create(
            name="Conta Corrente",
            description="Transações da conta corrente do banco: ABC",
            owner=User.objects.get(username='joao'))
        # Cria lancamentos
        Lancamento.objects.create(
            name="Padaria", date="2018-07-10", lanc_tp=2, value=20.00,
            owner=User.objects.get(username='joao'),
            conta=Conta.objects.get(name='Conta Corrente'))
        Lancamento.objects.create(
            name="Farmácia", date="2018-07-08", lanc_tp=2, value=65.00,
            owner=User.objects.get(username='joao'),
            conta=Conta.objects.get(name='Conta Corrente'))
        Lancamento.objects.create(
            name="Salário", date="2018-07-01", value=65.00,
            owner=User.objects.get(username='joao'),
            conta=Conta.objects.get(name='Conta Corrente'))

    def test_list_lancamentos(self):
        """ Verifica se está listando os lancamentos corretamente """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        date_ini = "2018-07-01"
        date_end = "2018-08-01"

        response = self.client.get(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': date_ini,
                            'end': date_end}), format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Lancamento.objects.count(), 3)
        self.assertEqual(Lancamento.objects.get_balance(
            date_ini=date_ini, date_end=date_end,
            conta_id=Conta.objects.get(
                owner__username='joao').pk), -20.00)

    def test_list_ii_lancamentos(self):
        """ Outro teste para ver se Lancamento.objects.get_balance funciona
        adequadamente """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        date_ini = "2018-07-07"
        date_end = "2018-08-01"

        response = self.client.get(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': date_ini,
                            'end': date_end}), format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Lancamento.objects.count(), 3)
        self.assertEqual(Lancamento.objects.get_balance(
            date_ini=date_ini, date_end=date_end,
            conta_id=Conta.objects.get(
                owner__username='joao').pk), -85.00)

    def test_list_iii_lancamentos(self):
        """ Outro teste diferente para Lancamento.objects.get_balance """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        date_ini = "2018-07-01"
        date_end = "2018-07-07"

        response = self.client.get(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': date_ini,
                            'end': date_end}), format='json')

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(Lancamento.objects.count(), 3)
        self.assertEqual(Lancamento.objects.get_balance(
            date_end=date_end,
            conta_id=Conta.objects.get(
                owner__username='joao').pk), 65.00)

    def test_list_forbiden_lancamentos(self):
        """ Verifica se está bloqueando a tentativa de listar objetos de outros
        usuários """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.get(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': "2018-07-01",
                            'end': "2018-08-01"}), format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_list_not_login_lancamentos(self):
        """ Verifica se está bloqueando a tentativa de listar objetos de
        usuários não logados """
        response = self.client.get(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': "2018-07-01",
                            'end': "2018-08-01"}), format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class DateFilterNewTest(APITestCase):
    """
    Módulo de teste para a criação de um novo DateFilterSerializer
    (lancamentos/serializers.py).
    Testa o LancamentoList view em lancamentos/views.py
    """
    def setUp(self):
        """ Cria o banco necessário para rodar os testes """
        # Cria usuários
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")
        # Cria contas
        Conta.objects.create(
            name="Conta Corrente", description="",
            owner=User.objects.get(username='joao'))
        # Cria lancamentos
        Lancamento.objects.create(
            name="Padaria", date="2018-07-10", lanc_tp=2, value=20.00,
            owner=User.objects.get(username='joao'),
            conta=Conta.objects.get(name='Conta Corrente'))
        Lancamento.objects.create(
            name="Farmácia", date="2018-08-08", lanc_tp=2, value=65.00,
            owner=User.objects.get(username='joao'),
            conta=Conta.objects.get(name='Conta Corrente'))
        # requests do dateFilter
        self.valid_payload = {
            'data_ini': "2018-07-01",
            'data_end': "2018-07-13"
        }
        self.invalid_payload = {
            'data_ini': "2018-07-07",
            'data_end': "2018-07-01"
        }

    def test_create_valid_date(self):
        """ Verifica se está sendo direcionado URL corretamente, a partir dos
        dados inseridos no form """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': "2018-08-01",
                            'end': "2018-08-30"}),
            self.valid_payload, format='json')

        self.assertEqual(response.status_code, status.HTTP_302_FOUND)

    def test_create_invalid_date(self):
        """ Verifica se está retornando má requisição se estiver tentando
        criar dateFilter não permitidos """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.post(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': "2018-08-01",
                            'end': "2018-08-30"}),
            self.invalid_payload, format='json')

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_not_login_date(self):
        """ Verifica se está bloqueando a tentativa de criar dateFilter
        por usuários não logados """
        response = self.client.post(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': "2018-08-01",
                            'end': "2018-08-30"}),
            self.valid_payload, format='json')

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_create_forbiden_date(self):
        """ Verifica se está bloqueando a tentativa de acesso por usuários
        não donos da conta """
        user = User.objects.get(username='maria')
        self.client.force_login(user)

        response = self.client.post(
            reverse('lancamento-list',
                    kwargs={'pk': Conta.objects.get(
                        owner__username='joao').pk,
                            'ini': "2018-08-01",
                            'end': "2018-08-30"}),
            self.valid_payload, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class UsuarioListAllTest(APITestCase):
    """
    Módulo de teste para a listagem de todos os usuários.
    Testa o UserList view em usuarios/views.py
    """
    def setUp(self):
        """ Cria o banco de dados """
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")

    def test_get_all_users(self):
        """ Listagem de todos os usuários """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('user-list'), format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(User.objects.count(), 2)


class UsuarioGetUnicTest(APITestCase):
    """
    Módulo de teste para selecionar um único usuário.
    Testa o UserDetail view em usuarios/views.py
    """
    def setUp(self):
        """ Cria o banco de dados """
        User.objects.create(
            username="joao", password="123456ab")
        User.objects.create(
            username="maria", password="123456ab")

    def test_get_valid_usuario(self):
        """ Verifica se está selecionando os usuários corretamente """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(
            reverse('user-detail',
                    kwargs={'pk': User.objects.get(username='joao').pk}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_invalid_usuario(self):
        """ Verifica se está retornando 403 quando se seleciona
        objetos não permitidos """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        response = self.client.get(reverse('user-detail', kwargs={'pk': 100}))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_forbiden_usuario(self):
        """ Verifica se está bloqueando a seleção por
        usuários sem permissão """
        user = User.objects.get(username='joao')
        self.client.force_login(user)

        self.client.login(username=user.username, password='123456ab')
        response = self.client.get(
            reverse('user-detail',
                    kwargs={'pk': User.objects.get(username='maria').pk}))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-08-01 18:35
from __future__ import unicode_literals

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Conta',
            fields=[
                ('id', models.AutoField(
                    auto_created=True,
                    primary_key=True,
                    serialize=False,
                    verbose_name='ID')),
                ('name', models.CharField(
                    max_length=255,
                    verbose_name='Título da Conta:')),
                ('description', models.TextField(
                    blank=True, default='', verbose_name='Descrição:')),
                ('owner', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='contas', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Lancamento',
            fields=[
                ('id', models.AutoField(
                    auto_created=True,
                    primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('name', models.CharField(
                    max_length=255,
                    verbose_name='Título do Lançamento:')),
                ('date', models.DateField(
                    default=datetime.date.today,
                    verbose_name='Data do Lançamento:')),
                ('lanc_tp', models.IntegerField(
                    choices=[(1, 'Débito'), (2, 'Crédito')],
                    default=1, verbose_name='Tipo de Lançamento:')),
                ('value', models.DecimalField(
                    decimal_places=2, max_digits=12,
                    verbose_name='Valor do Lançamento:')),
                ('conta', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='lancamentos',
                    to='contas_orama.Conta')),
                ('owner', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='lancamentos',
                    to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



"""
Views para a API
"""
from rest_framework import generics, permissions, mixins, status
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.decorators import api_view

from django.utils.timezone import now
from django.shortcuts import get_object_or_404
from django.http import HttpResponseRedirect
from django.contrib.auth.models import User

from contas_orama.models import Conta, Lancamento
from contas_orama.serializers import (ContaSerializer, LancamentoSerializer,
                                      UserSerializer, DateFilterSerializer, )
from contas_orama.permissions import IsOwner, IsUser
from contas_orama.pagination import LancamentoPagination


@api_view(['GET'])
def api_root(request):
    """
    Primeiro view da API (view raiz)
    """
    return Response({
        'contas': reverse('conta-list', request=request),
        'usuarios': reverse('user-list', request=request),
    })


class ContaList(generics.ListCreateAPIView):
    """
    View para criação e listagem de contas
    """
    queryset = Conta.objects.all()
    serializer_class = ContaSerializer

    # Permissão para que somente usuários autenticados consigam ter acesso
    # ao view de listagem de contas.
    permission_classes = (permissions.IsAuthenticated,)

    def create(self, request, pk=None):
        """
        Função create está sendo sobrescrita para que após a postagem seja
        possível redirecionar automaticamente para a url conta-list
        """
        serializer = ContaSerializer(data=request.data,
                                     context={'request': request})
        if serializer.is_valid():
            # Salva automaticamente o campo owner de utilizando o usuário
            # logado
            serializer.save(owner=self.request.user)

            # Redireiona após a postagem para a página da conta-list
            return HttpResponseRedirect(reverse('conta-list'))
        return Response(serializer.errors, status.HTTP_400_BAD_REQUEST)

    def get_queryset(self):
        """
        Filtra para que o usuário somente veja as próprias contas
        """
        return Conta.objects.filter(owner=self.request.user)


class ContaDetail(generics.RetrieveUpdateDestroyAPIView):
    """
    View que Retorna, Atualiza e Deleta uma conta
    Também retorna o link para acessar ver o extrato e
    fazer um novo lançamento.
    """
    queryset = Conta.objects.all()
    serializer_class = ContaSerializer

    # Permissão para que somente usuários logados e donos da conta
    # possam ter acesso a esta URL
    permission_classes = (permissions.IsAuthenticated, IsOwner,)

    def retrieve(self, request, pk=None):
        """
        Função retrieve foi sobrescrita para poder retornar um link funcional
        para lancamento-list
        """
        queryset = Conta.objects.all()
        conta = get_object_or_404(queryset, pk=pk)

        # Checa se o usuário tem permissão para acessar o objeto conta
        self.check_object_permissions(self.request, conta)
        serializer = ContaSerializer(conta, context={'request': request})

        # Define as datas: inicial e final para entrar na url de
        # lancamento-list.
        # Foi definido um intervalo padrão entre o dia atual,
        # e o primeiro dia do mês atual.
        data_ini_padrao = now().replace(day=1).strftime('%Y-%m-%d')
        data_end_padrao = now().strftime('%Y-%m-%d')

        update_dict = {'ver-extrato': reverse('lancamento-list',
                                              kwargs={'pk': pk,
                                                      'ini': data_ini_padrao,
                                                      'end': data_end_padrao},
                                              request=request),
                       'fazer-novo-lancamento': reverse('lancamento-create',
                                                        kwargs={'pk': pk},
                                                        request=request)}
        update_dict.update(serializer.data)

        return Response(update_dict)


class LancamentoCreate(mixins.RetrieveModelMixin,
                       mixins.CreateModelMixin,
                       generics.GenericAPIView):
    """
    View que permite a postagem de um novo lançamento utilizando
    a conta presente na URL
    """
    queryset = Lancamento.objects.all()
    serializer_class = LancamentoSerializer
    permission_classes = (permissions.IsAuthenticated, IsOwner,)

    def get(self, request, *args, **kwargs):
        """
        Chamada de retrieve
        """
        return self.retrieve(request, *args, **kwargs)

    def retrieve(self, request, pk=None):
        """
        Seleciona o objeto conta com o id dado pela URL
        """
        queryset = Conta.objects.all()
        conta = get_object_or_404(queryset, pk=pk)

        # Checa se o usuário tem as permissões necessárias para
        # acessar o objeto conta
        self.check_object_permissions(self.request, conta)
        serializer = ContaSerializer(conta, context={'request': request})
        return Response(serializer.data)

    def post(self, request, *args, **kwargs):
        """
        Chamadade create
        """
        return self.create(request, *args, **kwargs)

    def create(self, request, pk=None):
        """
        Cria o objeto lançamento
        """
        # Seleciona a conta onde o lancamento será realizados
        conta = get_object_or_404(Conta.objects.all(),
                                  pk=self.kwargs.get('pk'))

        # Checa se o usuário tem as permissões necessárias para
        # acessar o objeto conta
        self.check_object_permissions(self.request, conta)
        serializer = LancamentoSerializer(data=request.data,
                                          context={'request': request})
        if serializer.is_valid():
            # Salva o lancamento utilizando automaticamente o campo owner
            # e conta_id utilizando o request e a URL respectivamente
            serializer.save(owner=self.request.user,
                            conta_id=self.kwargs.get('pk'))

            # Redireciona após a postagem para a página da própria conta
            return HttpResponseRedirect(
                reverse('conta-detail',
                        kwargs={'pk': self.kwargs.get('pk')}))
        return Response(serializer.errors, status.HTTP_400_BAD_REQUEST)


class LancamentoDetail(generics.RetrieveUpdateDestroyAPIView):
    """
    View que Retorna, Atualiza e Deleta um lançamento
    """
    queryset = Lancamento.objects.all()
    serializer_class = LancamentoSerializer
    permission_classes = (permissions.IsAuthenticated, IsOwner,)


class LancamentoList(generics.ListCreateAPIView):
    """
    View que filtra todos os lançamentos feitos durante um período e em
    em uma daterminada conta, ambos parâmetros dados pela URL.

    Também retorna parâmetros julgados necessários para o front-end:
    o nome da conta, a data inicial e final do período e
    os saldos inicial, do período e o final.

    O view também permite a entrada de um novo período dado pelo usuário,
    redirecionando-o para uma URL com novas datas.
    """
    serializer_class = DateFilterSerializer
    permission_classes = (permissions.IsAuthenticated, IsOwner,)

    def list(self, request, pk=None, ini=None, end=None):
        """
        Lista os lançamentos da conta.pk = pk durante o período
        entre data = ini e data = end

        Mostra o saldo anterior e o final do período.
        """
        queryset = self.get_queryset()
        serializer = LancamentoSerializer(queryset,
                                          many=True,
                                          context={'request': request})

        # Preenchimento do dict com valores a serem utilizados na renderização
        # de lancamento-list
        # conta_nome - É o nome da conta à qual os lancamentos pertencem
        # inicio - É a data inicial da amostragem
        # fim - É a data final da amostragem
        # saldo_inicial - É o saldo da conta na data início
        # saldo_periodo - É o saldo da conta durante o período
        # saldo_final - É o saldo da conta na data fim
        saldo_inicio = Lancamento.objects.get_balance(
            date_end=self.kwargs['ini'],
            conta_id=self.kwargs['pk'])

        saldo_interv = Lancamento.objects.get_balance(
            date_ini=self.kwargs['ini'],
            date_end=self.kwargs['end'],
            conta_id=self.kwargs['pk'])

        conta_info = {'conta_nome': get_object_or_404(
            Conta.objects.all(), pk=self.kwargs['pk']).name,
                      'inicio': self.kwargs['ini'],
                      'fim': self.kwargs['end'],
                      'saldo_inicial': saldo_inicio,
                      'saldo_periodo': saldo_interv,
                      'saldo_fim': saldo_inicio + saldo_interv}

        # Retornar a lista de objetos 'paginada'
        # Paginação:
        # http://www.django-rest-framework.org/api-guide/pagination/#pagination
        # "Django provides a few classes that help you manage paginated data –
        # that is, data that’s split across several pages, with
        # “Previous/Next” links.""

        # get_paginated_response foi sobrescrita em contas/pagination.py
        # Sobrescrição feita para incluir o dict conta_info na paginação
        paginator = LancamentoPagination()
        page = paginator.paginate_queryset(serializer.data, request)
        if page is not None:
            return paginator.get_paginated_response(page, conta_info)
        return Response(serializer.data, status.HTTP_400_BAD_REQUEST)

    def get_queryset(self):
        """
        Filtra a lista de lançamentos utilizando as datas (inicial e final)
        e a conta dados na url
        """
        conta = get_object_or_404(Conta.objects.all(), pk=self.kwargs['pk'])

        # Checa se o usuário é dono da conta e tem permissão
        # para ter acesso a mesma.
        self.check_object_permissions(self.request, conta)
        ini = self.kwargs['ini']
        end = self.kwargs['end']
        return Lancamento.objects.filter(date__gte=ini,
                                         date__lte=end,
                                         conta=conta).order_by('date')

    def create(self, request, pk=None, ini=None, end=None):
        """
        Posta as novas data iniciais e finais
        redireciona para a URL desejada utilizando as datas
        """
        conta = get_object_or_404(Conta.objects.all(), pk=self.kwargs['pk'])
        # Checa se o usuário é dono da conta e tem permissão
        # para ter acesso a mesma.
        self.check_object_permissions(self.request, conta)

        serializer = DateFilterSerializer(data=request.data,
                                          context={'request': request})
        if serializer.is_valid():
            ini = serializer.validated_data['data_ini'].strftime('%Y-%m-%d')
            end = serializer.validated_data['data_end'].strftime('%Y-%m-%d')
            return HttpResponseRedirect(
                reverse('lancamento-list',
                        kwargs={'pk': self.kwargs.get('pk'),
                                'ini': ini,
                                'end': end}))

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class UserList(generics.ListAPIView):
    """
    View para a listagem dos usuários.
    No entanto, filtra para que os usuários somente possam ver a
    si mesmos.

    Poderia adicionar que superusuários pudessem ver todos.
    """
    queryset = User.objects.all()
    serializer_class = UserSerializer

    # Permissão para que somente usuários autenticados consigam ter
    # acesso ao view de listagem de usuários
    permission_classes = (permissions.IsAuthenticated,)

    def get_queryset(self):
        """
        Filtra para que o usuário somente veja a si próprio
        """
        return User.objects.filter(username=self.request.user).order_by('id')


class UserDetail(generics.RetrieveAPIView):
    """
    View para a visualização detalhada dos usuários.
    Usuário somente pode ver a própria página
    """
    queryset = User.objects.all()
    serializer_class = UserSerializer

    # Permissão para que somente usuários logados possam
    # acessar o próprio view
    permission_classes = (permissions.IsAuthenticated, IsUser,)



"""
Criado automaticamente pelo comando:
$ django-admin startapp contasOrama
"""
from django.contrib import admin
from contas_orama.models import Conta, Lancamento

admin.site.register(Conta)
admin.site.register(Lancamento)



"""
Paginação a ser utilizada em views.py
"""
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class LancamentoPagination(PageNumberPagination):
    """
    Paginação customizada para que fosse possível retornar um Response
    com todos os dados necessários para a renderização da URL
    lancamento-list
    """
    def get_paginated_response(self, data, ext_inf):
        return Response({
            'count': self.page.paginator.count,
            'next': self.get_next_link(),
            'previous': self.get_previous_link(),
            'results': data,
            'extra_info': ext_inf,
        })



"""
Criado automaticamente pelo comando:
$ django-admin startapp contasOrama
"""
from django.apps import AppConfig


class ContasoramaConfig(AppConfig):
    """
    Criado automaticamente pelo comando:
    $ django-admin startapp contasOrama
    """
    name = 'contas_orama'



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="contas_orama",
    version="0.1.0",
    author="Felippe",
    author_email="felippemsc@gmail.com",
    description="API para gestão de contas financeiras",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/felippemsc/teste_orama",
    packages=setuptools.find_packages(),
    classifiers=(
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        "License :: OSI Approved :: MIT License",
    ),
)
